/*
 * IPK export plugin for Eclipse. 
 * Copyright (C) 2005  Bernd Lachner, InSo GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * You can reach the authr via mail:
 * 
 * InSo GmbH, Individuelle Software-Erstellung
 * Hannongstrae 5
 * 67227 Frankenthal, Germany
 * 
 * or electronic mail:
 * 
 * info@inso-gmbh.de
 */

package de.inso_gmbh.io;


import java.io.*;

/**
 * Class for a tar output stream.
 * 
 * @author Bernd Lachner, InSo GmbH
 */
public class TarOutputStream extends FilterOutputStream
{

	/**
	 * Constructor.
	 * 
	 * @param oOutputStream Underlying stream to output the tar stream. E.g. a GZIPOutputStream for packed tar files.
	 */
	public TarOutputStream(OutputStream oOutputStream)
	{
		super(oOutputStream);
	}
	
	private TarEntry m_oActiveEntry = null;
	private ByteArrayOutputStream m_oEntryData = null;

	/**
	 * Puts a new entry in the stream. The write methods will write data to this entry,
	 * until the closeEntry method is called.
	 * 
	 * @param oNewEntry New tar entry.
	 */	
	public void putNextEntry(TarEntry oNewEntry) throws IOException
	{
		m_oActiveEntry = oNewEntry;
		m_oEntryData = new ByteArrayOutputStream();
	}
	
	/**
	 * Writes data to the end of the actual tar entry.
	 * 
	 * @param abData Byte array which should be appended to the actual tar entry..
	 * @throws IOException
	 */
	public void write(byte[] abData) throws IOException
	{
		m_oEntryData.write(abData);	
	}

	/**
	 * Writes a given number of bytes from a given position in a byte array to the actual tar entry.
	 * 
	 * @param abData Byte array from which bytes should be written to the tar entry.
	 * @param iOffset Offset in abData from which bytes should be taken.
	 * @param iLen Number of bytes which should be taken from abData.
	 * @throws IOException
	 */
	public void write(byte[] abData, int iOffset, int iLen) throws IOException
	{
		m_oEntryData.write(abData, iOffset, iLen);
	}
	
	/** 
	 * Writes a single byte to the tar entry at the actual position.
	 * 
	 * @param iData Data to write.
	 * @throws IOException
	 * */
	public void write(int iData) throws IOException
	{
		m_oEntryData.write(iData);
	}
	
	/**
	 * Closes the actual tar entry. No data can be writen after this method is called.
	 */
	public void closeEntry() throws IOException
	{
		int iFillBytes = 0;
		if (m_oEntryData.size() > 0)
		{
		 	iFillBytes = TarEntry.BLOCK_SIZE - (m_oEntryData.size() % TarEntry.BLOCK_SIZE);
		}
		m_oActiveEntry.setSize(m_oEntryData.size());
		out.write(m_oActiveEntry.getHeader());		
		m_oEntryData.writeTo(out);
		while (iFillBytes > 0)
		{
			out.write(0);
			iFillBytes--;
		}
		m_oActiveEntry = null;	
	}

	/**
	 * Closes the tar archive tar entry. Writes the "end of archive" blocks. 
	 * No data can be writen after this method is called.
	 */	
	public void closeArchive() throws IOException
	{
		// Write to blocks with 0 bytes
		int iFillBytes = 1024;
		while (iFillBytes > 0)
		{
			out.write(0);
			iFillBytes--;
		}		
	}
	
	/**
	 * Closes the stream and writes the "end of archive" blocks.
	 */
	public void close() throws IOException
	{
		closeArchive();
		super.close();
	}
}

