/*
 * IPK export plugin for Eclipse. 
 * Copyright (C) 2005  Bernd Lachner, InSo GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * You can reach the authr via mail:
 * 
 * InSo GmbH, Individuelle Software-Erstellung
 * Hannongstrae 5
 * 67227 Frankenthal, Germany
 * 
 * or electronic mail:
 * 
 * info@inso-gmbh.de
 */

package de.inso_gmbh.ipkexportplugin.wizards;

import java.beans.Expression;
import java.beans.Statement;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Iterator;

import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.wizard.WizardPage;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.ModifyEvent;
import org.eclipse.swt.events.ModifyListener;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * The base class for all ipk wizard pages.
 * 
 * @author Bernd Lachner, InSo GmbH
 *
 */
public abstract class ExtWizardPage extends WizardPage 
{
	protected IpkExportWizard m_oParent = null;

	protected static final char m_acAllowedUnixFileChar[] 
	= { 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', '.', '-', '_' };
	
	 
	/**
	 * Constructor.
	 * 
	 * @param sPageName The name of the page.
	 */
	protected ExtWizardPage(String sPageName, IpkExportWizard oParent)
	{
		super(sPageName);
		m_oParent = oParent;
	}
	
	/**
	 * Constructor.
	 * 
	 * @param sPageName The name of the page.
	 * @param sTitle The title for this wizard page, or null if none.
	 * @param oTitleImage The image descriptor for the title of this wizard page, or null if none.
	 */
	protected ExtWizardPage(String sPageName, String sTitle, ImageDescriptor oTitleImage, IpkExportWizard oParent)
	{
		super(sPageName, sTitle, oTitleImage);
		m_oParent = oParent;
	}
	
	/**
	 * Creates a new composite widget with a grid layout.
	 * 
	 * @param oParent The parent widget to insert the new grid container.
	 * @param iColumns Number of columns of the grid.
	 * @return Composite widget with the grid layout.
	 */
	protected Composite makeGridContainer(Composite oParent, int iColumns)
	{
		GridData oGridData = new GridData(GridData.FILL_HORIZONTAL);
		Composite oContainer = new Composite(oParent, SWT.NULL);
		oContainer.setLayoutData(oGridData);
		GridLayout oLayout = new GridLayout();
		oContainer.setLayout(oLayout);
		oLayout.numColumns = iColumns;
		return oContainer;
	}
	
	/**
	 * Adds a label with a following text widget to a container with a grid layout.
	 *  
	 * @param oContainer Containert to add the widgets.
	 * @param sLabelText The text for the label widget.
	 * @param iColumns The number of columns the text widget should occupy.
	 * @param bReadOnly If true, the text widget is read only.
	 * @return The new text widget.
	 */
	protected Text addLabelAndText(Composite oContainer, String sLabelText, int iColumns, boolean bReadOnly)
	{
		Label oLabel = new Label(oContainer, SWT.NULL);
		oLabel.setText(sLabelText);
		Text oText = null;
		if (bReadOnly == true)
		{
			oText = new Text(oContainer, SWT.BORDER | SWT.SINGLE | SWT.READ_ONLY);
		}
		else
		{
			oText = new Text(oContainer, SWT.BORDER | SWT.SINGLE);
			oText.addModifyListener(new ModifyListener() 
			{
				public void modifyText(ModifyEvent oModifyEvent) 
				{
					dialogChanged();
				}
			});
		}
		GridData oGridData = new GridData(GridData.FILL_HORIZONTAL);
		oGridData.horizontalSpan = iColumns;
		oText.setLayoutData(oGridData);
		return oText;
	}
	
	/**
	 * Checks if the text in a text widget is empty (no characters or only spaces).
	 * 
	 * @param oText The text widget to check.
	 * @return True if empty, otherwise false.
	 */
	protected boolean isTextEmpty(Text oText)
	{
		if (oText.getText().length() > 0)
		{
			String sText = oText.getText();
			for (int i = 0; i < sText.length(); i++)
			{
				if (sText.charAt(i) != ' ')
					return false;
			}
		}
		return true;
	}
	
	/**
	 * Adds a label with a following combo widget to a container with a grid layout.
	 * 
	 * @param oContainer Container to add the widgets.
	 * @param sLabelText The text for the label widget.
	 * @param asComboEntries String array with the combo box entries.
	 * @return The new combo widget.
	 */
	protected Combo addLabelAndCombo(Composite oContainer, String sLabelText, String asComboEntries[])
	{
		Label oLabel = new Label(oContainer, SWT.NULL);
		oLabel.setText(sLabelText);
		Combo oCombo = new Combo(oContainer, SWT.READ_ONLY | SWT.BORDER | SWT.SINGLE);
		GridData oGridData = new GridData(GridData.FILL_HORIZONTAL);
		oCombo.setLayoutData(oGridData);
		for (int i = 0; i < asComboEntries.length; i++)
		{
			oCombo.add(asComboEntries[i]);
		}
		oCombo.select(0);
		oCombo.addModifyListener(new ModifyListener() 
		{
			public void modifyText(ModifyEvent oModifyEvent) 
			{
				dialogChanged();
			}
		});
		return oCombo;
	}
	
	/**
	 * Gets a string value from a string array related to the combo box index.
	 * 
	 * @param oCombo The combo box to get the index.
	 * @param asComboValues The string array to get the value.
	 * @return The value related to the combo box index or a empty string if no value in the string array for the index.
	 */	
	protected String getComboValue(Combo oCombo, String asComboValues[])
	{
		if (oCombo.getSelectionIndex() < 0 || oCombo.getSelectionIndex() >= asComboValues.length)
		{
			return "";
		}
		else
		{
			return asComboValues[oCombo.getSelectionIndex()];
		}
	}

	/**
	 * Sets the combo box index related to a string in a string array.
	 * 
	 * @param oCombo The combo box which index should be set.
	 * @param sValue Th string value.
	 * @param asComboValues The string array which all values related to a combo box index.
	 */
	protected void setComboValue(Combo oCombo, String sValue, String asComboValues[])
	{
		for (int i = 0; i < asComboValues.length; i++)
		{
			if (asComboValues[i].equals(sValue) && i < oCombo.getItemCount())
			{
				oCombo.select(i);
				break;
			}
		}
	}
	
	/**
	 * A verify handle method for a text widget. It verifies if the text of the widget  
	 * contains allowed filename.
	 * 
	 * This verify handle method only allows the following charcters:
	 * 0..9, a..z, A..Z, ., _, -
	 * 
	 * Actual file systems on Linux, windows, etc. allows more characters, but this should 
	 * be a secure methode to check if a filename is valid on most systems.
	 * 
	 * @param oVerifyEvent Verify event.
	 */
	protected void verifyFileName(VerifyEvent oVerifyEvent) 
	{
		if (oVerifyEvent.text.length() > 0)
		{
			// oVerifyEvent.character doesn't work with the SWT Linux GTK version it is always empty
			// therefore oVerifyEvent.text is used
			char cVerify = Character.toLowerCase(oVerifyEvent.text.charAt(0));
			if (Character.isISOControl(cVerify))
			{
				oVerifyEvent.doit = true;
				return;
			}
			for (int i = 0; i < m_acAllowedUnixFileChar.length; i++)
			{
				if (cVerify == m_acAllowedUnixFileChar[i])
				{
					oVerifyEvent.doit = true;
					return;
				}
			}
			oVerifyEvent.doit = false;			
		}
		else
		{
			oVerifyEvent.doit = true;
		}
	}

	/**
	 * Sets the status of the wizard page.
	 *  
	 * @param sErrorMessage Error message or null if no error. If null the wizard page is marked as completed. 
	 */
	protected void updateStatus(String sErrorMessage) 
	{
		setErrorMessage(sErrorMessage);
		setPageComplete(sErrorMessage == null);
	}

	/**
	 * Method must be implemented from subclasses. 
	 * Method is called if a combo box or text box value is changed.
	 */
	protected abstract void dialogChanged();
	
	/**
	 * Reads a value from a XML node.
	 * 
	 * @param sKey The name of the XML node for which the value shopuld be returned.
	 * @return The value of the node or "" if no node is found.
	 */
	protected String getXMLSettingValue(String sKey)
	{
		if (m_oParent.getDocumentIPKSettings() != null)
		{
			Node oRootNode = m_oParent.getDocumentIPKSettings().getDocumentElement();
			if (oRootNode != null)
			{
				NodeList listChildNodes = oRootNode.getChildNodes();
				for (int i = 0; i < listChildNodes.getLength(); i++)
				{
					Node oChildNode = listChildNodes.item(i); 
					if (oChildNode.getNodeName() == null)
						continue;
					if (oChildNode.getNodeName().equals(sKey))
					{
						// The next 4 lines can be replaced by "return oChildNode.getTextContent();"
						// if DOM Level 3 / Java 2 SE 5 is used.
						Node oTextNode = oChildNode.getFirstChild();
						if (oTextNode != null && oTextNode.getNodeType() == Node.TEXT_NODE)
						{
							return oTextNode.getNodeValue();
						}												
					}
				}
			}
		}
		return "";
	}
	
	/**
	 * Creates a new node and sets the value.
	 * 
	 * @param oParentNode Parent of the new node.
	 * @param sKey Name of the new node.
	 * @param sValue Value of the new node.
	 */
	protected void setXMLSettingValue(Node oParentNode, String sKey, String sValue)
	{
		if (m_oParent.getDocumentIPKSettings() != null)
		{
			Node oChildNode = m_oParent.getDocumentIPKSettings().createElement(sKey);
			org.w3c.dom.Text oTextNode = m_oParent.getDocumentIPKSettings().createTextNode(sValue);
			oChildNode.appendChild(oTextNode);
			oParentNode.appendChild(oChildNode);
		}
	}
	
	/**
	 * Creates a new node as child of the root node and sets the value.
	 * 
	 * @param sKey Name of the new node.
	 * @param sValue Value of the new node.
	 */
	protected void setXMLSettingValue(String sKey, String sValue)
	{
		if (m_oParent.getDocumentIPKSettings() != null)
		{
			Node oRootNode = m_oParent.getDocumentIPKSettings().getDocumentElement();
			if (oRootNode == null)
			{
				oRootNode = m_oParent.getDocumentIPKSettings().createElement("IPKSettings");
				m_oParent.getDocumentIPKSettings().appendChild(oRootNode);
			}				
			setXMLSettingValue(oRootNode, sKey, sValue);
		}
	}

	/**
	 * Reads the values of all getter/setter methods of a given object from 
	 * a XML file and calls the setter methods with this value. At the moment
	 * only String and Integer getter/setter methods are supported.
	 * 
	 * @param oObject Object which values of the getter/setter methods should be read from the XML file.
	 */
	protected void readSettingValuesFromXML(Object oObject)
	{
		ArrayList listGetSetFields = getGetterSetterList(oObject);
		Iterator it = listGetSetFields.iterator();
		while (it.hasNext())
		{
			GetterSetterDescription oMethodDescr =  (GetterSetterDescription) it.next();
			try
			{
				Object oParameter = null; 
				if (oMethodDescr.getParameterType().isAssignableFrom(String.class))
				{
					oParameter = getXMLSettingValue(oMethodDescr.getMethodName());					
				}
				else if (oMethodDescr.getParameterType().isAssignableFrom(Integer.TYPE))
				{
					oParameter = Integer.decode(getXMLSettingValue(oMethodDescr.getMethodName()));
				}
				if (oParameter != null)
				{
					Statement oStatement = new Statement(oObject, "set" + oMethodDescr.getMethodName(), new Object[] { oParameter });
					oStatement.execute();
				}
			}
			catch (Exception e)
			{
				// Ignore exceptions. If exceptions appear data just will not be restored from XML setting file. 
			}
		}
	}
		
	/**
	 * Writes the values of all getter/setter methods of a given object to 
	 * a XML file by calling the getter methods to get the value. At the moment
	 * only String and Integer getter/setter methods are supported.
	 * 
	 * @param oObject Object which values of the getter/setter methods should be write to the XML file.
	 */
	protected void writeSettingValuesToXML(Object oObject)
	{
		ArrayList listGetSetFields = getGetterSetterList(oObject);
		Iterator it = listGetSetFields.iterator();
		while (it.hasNext())
		{
			GetterSetterDescription oMethodDescr = (GetterSetterDescription) it.next();
			Expression oExpression = new Expression(oObject, "get" + oMethodDescr.getMethodName(), null);
			try
			{
				if (oExpression.getValue() instanceof String)
				{
					setXMLSettingValue(oMethodDescr.getMethodName(), (String) oExpression.getValue());
				}
				else if (oExpression.getValue() instanceof Integer)
				{
					setXMLSettingValue(oMethodDescr.getMethodName(), ((Integer)oExpression.getValue()).toString());
				}		
			}
			catch (Exception e)
			{
				// Ignore exceptions. If exceptions appear data just will not be saved to XML setting file.
			}
		}
	}
	
	/**
	 * 
	 * Class for describing getter/setter methods.
	 *
	 */
	private class GetterSetterDescription
	{
		private Class m_oParameterType;
		public String m_sMethodName;

		/**
		 * Constructor.
		 * 
		 * @param sMethodName Name of the getter/setter method without the get/set.
		 * @param oParameterType Parameter type of the getter/setter method.
		 */
		GetterSetterDescription(String sMethodName, Class oParameterType)
		{
			m_sMethodName = sMethodName;
			m_oParameterType = oParameterType;
		}
		
		/**
		 * Returns the Parameter type of the getter/setter method.
		 * 
		 * @return Parameter type of the getter/setter method.
		 */
		public Class getParameterType()
		{
			return m_oParameterType;
		}
		
		/**
		 * Returns the Name of the getter/setter method.
		 * 
		 * @return Name of the getter/setter method without the get/set.
		 */
		public String getMethodName()
		{
			return m_sMethodName;
		}		
	}

	/**
	 * Returns the getter/setter methods of a given object. 
	 * 
	 * @param oObject The object for which the getter/setter methods should be found.  
	 * @return List with descriptions (GetterSetterDescription objects) of the getter/setter methods of a given class.
	 */
	protected ArrayList getGetterSetterList(Object oObject)
	{
		ArrayList listGetSetFields = new ArrayList();
		ArrayList listIncludedMethods = new ArrayList(); 
		
		Method[] oMethods = oObject.getClass().getMethods();
		for (int i = 0; i < oMethods.length; i++)
		{
			if (oMethods[i].getDeclaringClass().getName().equals(oObject.getClass().getName()))
			{
				String sSearchMethod = oMethods[i].getName();
				Class oParameterType = null;
				if (sSearchMethod.startsWith("set") == true)
				{
					sSearchMethod = "get" + sSearchMethod.substring(3);
				}
				else if (sSearchMethod.startsWith("get") == true)
				{
					sSearchMethod = "set" + sSearchMethod.substring(3);
					oParameterType = oMethods[i].getReturnType();
				}
				else
				{
					// Not a get/set method
					continue;
				}
				for (int j = 0; j < oMethods.length; j++)
				{
					if (oMethods[j].getName().equals(sSearchMethod))
					{
						if (sSearchMethod.startsWith("get"))
						{
							oParameterType = oMethods[j].getReturnType();
						}
						sSearchMethod = sSearchMethod.substring(3);
						if (listIncludedMethods.contains(sSearchMethod) == false)
						{
							listIncludedMethods.add(sSearchMethod);
							listGetSetFields.add(new GetterSetterDescription(sSearchMethod, oParameterType));
						}
						break;
					}
				}
			}
		}
		return listGetSetFields;
	}
	
	/**
	 * Method must be implemeted from subclasses.
	 * Method is called when the dialog settings should be saved to the project reosurece / XML file.
	 */
	protected abstract void writeToProjectResources();
}
