/*
 * IPK export plugin for Eclipse. 
 * Copyright (C) 2005  Bernd Lachner, InSo GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * You can reach the authr via mail:
 * 
 * InSo GmbH, Individuelle Software-Erstellung
 * Hannongstrae 5
 * 67227 Frankenthal, Germany
 * 
 * or electronic mail:
 * 
 * info@inso-gmbh.de
 */

package de.inso_gmbh.ipkexportplugin.wizards;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.layout.*;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.*;

import de.inso_gmbh.ipkexportplugin.IpkExportPlugin;

import java.io.*;

/**
 * The "control settings" wizard page allows setting the parameters of the ipk control file.
 * 
 * @author Bernd Lachner, InSo GmbH
 *
 */
public class IpkExportWizardPageControlSettings extends ExtWizardPage 
{
	private Text m_oPackageNameText = null;
	private Text m_oPackageVersionText = null; 
	private Text m_oPackageDescriptionText = null;
	private Text m_oPackageMaintainerText = null;
	private Text m_oPackageDependsText = null;
	private Combo m_oPriorityCombo = null;
	private Combo m_oSectionCombo = null;
	private Combo m_oArchitectureCombo = null;
	private Text m_oIpkName = null;
	private Text m_oIpkDirText = null;
	
	private static final String m_asPriority[] = {"required", "standard", "important", "optional", "extra"};
	private static final String m_asPriorityCombo[] = {"required", "standard", "important", "optional", "extra"};
	private static final String m_asSection[] = { "admin", "base", "comm", "editors", "extras", "graphics", "libs", "misc", "net", "text", "web", "x11"}; 			
	private static final String m_asSectionCombo[] = { "admin", "base", "comm", "editors", "extras", "graphics", "libs", "misc", "net", "text", "web", "x11"}; 			
	private static final String m_asArchitecture[] = { "all", "arm"}; 			
	private static final String m_asArchitectureCombo[] = { "all", "arm"}; 			
	
	private boolean m_bIsCreated = false;
	
	/**
	 * Constructor for "control settings" wizard page.
	 * 
	 * @param oParent Reference to the parent wizard object.
	 */
	public IpkExportWizardPageControlSettings(IpkExportWizard oParent) 
	{
		super("ControlSettingsPage", "IPK Export / Control settings", IpkExportPlugin.createImageDescriptor("icons/ipk_export_wizard.gif"), oParent);
		setDescription("The parameters of the control file for the IPK package.");
	}

	/**
	 * @see IDialogPage#createControl(Composite)
	 */
	public void createControl(Composite parent) 
	{
		Composite container = new Composite(parent, SWT.NULL);
		GridLayout layout = new GridLayout();
		container.setLayout(layout);
		layout.verticalSpacing = 9;

		Label label = new Label(container, SWT.NULL);
		label.setText("Pacakge description:");
		this.createControlSettingsSelectionGroup(container);
		label = new Label(container, SWT.NULL);		
		label.setText("Select the export destination:");
		this.createIPKSelectionGroup(container);
		this.m_bIsCreated = true;

		readSettingValuesFromXML(this);
		
		dialogChanged();
		setControl(container);
	}
	
	/**
	 * Creates the widgets for the control file parameters. 
	 * 
	 * @param oParent Parent widget.
	 */
	private void createControlSettingsSelectionGroup(Composite oParent)
	{
		Composite container = makeGridContainer(oParent, 2);
		
		this.m_oPackageNameText = addLabelAndText(container, "Package name:", 1, false);
		this.m_oPackageNameText.addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent oVerifyEvent)
			{
				verifyFileName(oVerifyEvent);
			}
		});
		this.m_oPackageVersionText = addLabelAndText(container, "Package version:", 1, false);
		this.m_oPackageVersionText.addVerifyListener(new VerifyListener()
		{
			public void verifyText(VerifyEvent oVerifyEvent)
			{
				verifyFileName(oVerifyEvent);
			}
		});
		this.m_oPackageDescriptionText = addLabelAndText(container, "Package description:", 1, false);
		this.m_oPackageMaintainerText = addLabelAndText(container, "Package maintainer:", 1, false);
		this.m_oPackageDependsText = addLabelAndText(container, "Package depends on:", 1, false);
		this.m_oArchitectureCombo = addLabelAndCombo(container, "Architecture:", IpkExportWizardPageControlSettings.m_asArchitectureCombo);
		this.m_oPriorityCombo = addLabelAndCombo(container, "Priority:", IpkExportWizardPageControlSettings.m_asPriorityCombo); 
		this.m_oPriorityCombo.select(3);
		this.m_oSectionCombo = addLabelAndCombo(container, "Section:", IpkExportWizardPageControlSettings.m_asSectionCombo);
		this.m_oSectionCombo.select(4);
		this.m_oIpkName = addLabelAndText(container, "Ipk name:", 1, true);
	}
	
	/**
	 * Creates the widgets for the ipk destination directory selection.
	 *
	 * @param oParent Parent widget.
	 */
	private void createIPKSelectionGroup(Composite oParent)
	{
		Composite oContainer = makeGridContainer(oParent, 3);

		this.m_oIpkDirText = addLabelAndText(oContainer, "IPK dir:", 1, false);
		this.m_oIpkDirText.setText(System.getProperty("user.home"));
		Button button = new Button(oContainer, SWT.PUSH);
		button.setText("Browse...");
		button.addSelectionListener(new SelectionAdapter() 
		{
			public void widgetSelected(SelectionEvent e) 
			{
				handleIpkDirSelection();
			}
		});
	}

	/**
	 * Gets the name parameter for the control file.
	 * 
	 * @return Name.
	 */
	public String getPackageName()
	{
		return this.m_oPackageNameText.getText();
	}	

	/**
	 * Sets the name parameter for the control file.
	 * 
	 * @param sPackageName Name of the package.
	 */
	public void setPackageName(String sPackageName)
	{
		this.m_oPackageNameText.setText(sPackageName);
	}

	
	/**
	 * Gets the version parameter for the control file.
	 * 
	 * @return Version.
	 */
	public String getPackageVersion()
	{
		return this.m_oPackageVersionText.getText();
	}	

	/**
	 * Sets the version parameter for the control file.
	 * 
	 * @param sPackageVersion Version of the package.
	 */
	public void setPackageVersion(String sPackageVersion)
	{
		this.m_oPackageVersionText.setText(sPackageVersion);
	}	
	
	/**
	 * Gets the description parameter for the control file.
	 * 
	 * @return Description.
	 */
	public String getPackageDescription()
	{
		return this.m_oPackageDescriptionText.getText();
	}

	/**
	 * Sets the description parameter for the control file.
	 * 
	 * @param sPackageDescription Description of the package.
	 */
	public void setPackageDescription(String sPackageDescription)
	{
		this.m_oPackageDescriptionText.setText(sPackageDescription);
	}

	/**
	 * Gets the maintainer parameter for the control file.
	 * 
	 * @return Maintainer.
	 */
	public String getPackageMaintainer()
	{
		return this.m_oPackageMaintainerText.getText();
	}	
	
	/**
	 * Sets the maintainer parameter for the control file.
	 * 
	 * @param sPackageMaintainer Package maintainer.
	 */
	public void setPackageMaintainer(String sPackageMaintainer)
	{
		this.m_oPackageMaintainerText.setText(sPackageMaintainer);
	}		

	/**
	 * Gets the depends parameter for the control file.
	 * 
	 * @return Depends.
	 */
	public String getPackageDepends()
	{
		return this.m_oPackageDependsText.getText();
	}

	/**
	 * Sets the depends parameter for the control file.
	 * 
	 * @param sPackageDepends Package dependencies.
	 */
	public void setPackageDepends(String sPackageDepends)
	{
		this.m_oPackageDependsText.setText(sPackageDepends);
	}

	/**
	 * Gets the architecture parameter for the control file.
	 * 
	 * @return Architecture.
	 */
	public String getPackageArchitecture()
	{
		return getComboValue(this.m_oArchitectureCombo, IpkExportWizardPageControlSettings.m_asArchitecture);
	}

	/**
	 * Gets the architecture parameter for the control file.
	 * 
	 * @param sPackageArchitecture Architecture which is supported by the package.
	 */
	public void setPackageArchitecture(String sPackageArchitecture)
	{
		setComboValue(this.m_oArchitectureCombo, sPackageArchitecture, IpkExportWizardPageControlSettings.m_asArchitecture);
	}

	/**
	 * Gets the priority parameter for the control file.
	 * 
	 * @return Priotrity.
	 */
	public String getPackagePriority()
	{
		return getComboValue(this.m_oPriorityCombo, IpkExportWizardPageControlSettings.m_asPriority);
	}

	/**
	 * Sets the priority parameter for the control file.
	 * 
	 * @param sPackagePriority Package priority.
	 */
	public void setPackagePriority(String sPackagePriority)
	{
		setComboValue(this.m_oPriorityCombo, sPackagePriority, IpkExportWizardPageControlSettings.m_asPriority);
	}

	/**
	 * Gets the section parameter for the control file.
	 * 
	 * @return Section.
	 */
	public String getPackageSection()
	{
		return getComboValue(this.m_oSectionCombo, IpkExportWizardPageControlSettings.m_asSection);
	}

	/**
	 * Sets the section parameter for the control file.
	 * 
	 * @param sPackageSection Package section.
	 */
	public void setPackageSection(String sPackageSection)
	{
		setComboValue(this.m_oSectionCombo, sPackageSection, IpkExportWizardPageControlSettings.m_asSection);
	}

	/**
	 * Gets the Ipk save directory.
	 * 
	 * @return directory path
	 */
	public String getIpkDir()
	{
		return m_oIpkDirText.getText();
	}
	
	/**
	 * sets the Ipk save directory.
	 * 
	 * @param sIpkDir directory path
	 */
	public void setIpkDir(String sIpkDir)
	{
		if (sIpkDir.length() > 0)
		{
			// Only accept directory pathes which are not empty
			m_oIpkDirText.setText(sIpkDir);
		}
	}
	
	/**
	 * Gets the name of the ipk package which sould be generated. The name is build as follows:
	 * 
	 * Name_Version_Architecture.ipk
	 * 
	 * @return Name of the ipk package.
	 */
	public String getIpkName()
	{
		return this.m_oIpkName.getText();
	}
		
	/**
	 * Gets the name of the ipk package including the full destination path.
	 * 
	 * @return Name of the ipk package including destination path.
	 */
	public String getIpkFileName()
	{
		String sDir = this.m_oIpkDirText.getText().trim();
		if (sDir.endsWith(File.separator) == false)
			sDir += File.separator; 
		return (sDir + this.m_oIpkName.getText());
	}
	

	/**
	 * Handle methode for the ipk destination dir browse button.
	 */
	private void handleIpkDirSelection() 
	{
		DirectoryDialog oSaveDirDialog = new DirectoryDialog(getShell());
		oSaveDirDialog.setMessage("Choose IPK file destination directory");
		String sSaveDirName = oSaveDirDialog.open();
		if (sSaveDirName != null)
		{
			this.m_oIpkDirText.setText(sSaveDirName);
		}
	}

	/**
	 * Is called when the dialog settings have changed.
	 */	
	protected void dialogChanged() 
	{
		if (m_bIsCreated == false)
			return;

		// Generate Ipk name
		this.m_oIpkName.setText((getPackageName() + "_" + getPackageVersion() + "_" + getPackageArchitecture() + ".ipk").toLowerCase());

		// Check if all required fields are set			
		if (isTextEmpty(this.m_oPackageNameText) || isTextEmpty(this.m_oPackageVersionText) ||
			isTextEmpty(this.m_oPackageMaintainerText) || isTextEmpty(this.m_oPackageDescriptionText))
		{
			updateStatus("Package, Version, Architecture, Maintainer, and Description are required!");
			return;
		}
		
		// Check if ipk destination directory is a valid directory
		File oCheckDir = new File(this.m_oIpkDirText.getText());
		if (oCheckDir.isDirectory() == false)
		{
			updateStatus("Ipk destination directory is not valid!");
			return;
		}
		updateStatus(null);
	}
	
	/**
	 * @see ExtWizardPage#writeToProjectResources() 
	 */
	protected void writeToProjectResources()
	{
		writeSettingValuesToXML(this);		
	}
}